#include <windows.h>
#include <math.h> //Floating point operations
#include "ConfigForm.h"

//Graphics Engine Enitialization
	//Which graphics libraries to compile into the project
	#define UseOpenGL TRUE
	#define UseWinGDI TRUE

	//If including OpenGL...
	#if UseOpenGL
		#include <gl/gl.h>
		#pragma comment(lib, "opengl32")
	#endif

	//Current Graphics Engine
	enum GraphicsEngines {GE_WinGDI, GE_OpenGL};
	GraphicsEngines GraphicsEngine=GE_OpenGL;

	//Graphics Buffers are one of the 2 types
	union GraphicsBuffer
	{
		#if UseOpenGL
			GLuint OGL; //Holds back buffer in a texture, and during screen draw, it draws the texture as a quad to the real back buffer, and then swaps that surface to the primary buffer.
		#endif
		HDC GDI; //Backbuffer is a graphics device context
	};

//Default Parameters
WORD Width=600, Height=400; //Window Dimensions
double XMin=-2, XMax=1, YMin=-1, YMax=1; //The X/Y axis min/max (boundary) points
int MaxIterations=200; //Detail Level
int AnimationFrames=120; //Number of frames to draw a zoom animation in
bool AnimationEnabled=true; //Whether to animate moving or not

//Julia fractal configuration parameters
bool JUseC=true;
double JReal=0, JImag=0;

//Program specific global variables
GraphicsBuffer MainBackBuffer; //This holds the current fractal level
HWND TheMainHwnd, DialogHwnd;

//Function headers
	//Window graphics device functions
	GraphicsBuffer CreateGraphicsBuffer(HWND CopyHwnd); //Creates a graphics buffer
	void ReleaseGraphicsBuffer(GraphicsBuffer GBuffer); //Releases a graphics buffer
	void SizeGraphicsBuffer(HWND CopyHwnd, GraphicsBuffer GBuffer); //Sizes a graphic buffer's surface to the needed size for the window
	void DrawToWindow(HWND hWnd); //Draw the back buffer to the main screen
	void AnimateZoom(double NewXMin, double NewXMax, double NewYMin, double NewYMax, HWND hWnd, int Frames); //Animate a zoom in/out. Takes in new boundary points, the window to draw to, and the number of frames to do the animation in
	LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam); //Process windows events
	bool InitializeOpenGL(HWND TheHwnd); //Initialize OpenGL and Back Buffer
	//Fractal functions
	void CreateFractal(); //Draws the fractal into the back buffer
	void DrawFractal(DWORD *PixelArray); //Does the actual fractal drawing into a pixel array
	DWORD ProcessColor(int IterationsToEscape, double ValueAtEndReal, double ValueAtEndImaginary, double BailoutRadius, double Power); //Determine the color of a pixel based on its state	

int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR pCmdLine,int nCmdShow)
{
	//Create the window class
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), NULL, WndProc, 0L, 0L, hInst, LoadIcon(hInst, "MAIN_ICON"), LoadCursor(NULL,IDC_ARROW), NULL, NULL, "JuliaFractal", NULL };
	if(RegisterClassEx(&wc)==0)
		return false;

	//Determine the size of the actual window that we want based upon client area dimensions
	RECT WindowRect={0, 0, Width, Height};
	AdjustWindowRect(&WindowRect, WS_OVERLAPPEDWINDOW, false);

	//Create the window and display it
	HWND MainHwnd=TheMainHwnd=CreateWindowEx(NULL, "JuliaFractal", "Julia Fractal", WS_OVERLAPPEDWINDOW, 0, 0, WindowRect.right-WindowRect.left, WindowRect.bottom-WindowRect.top, NULL, NULL, hInst, NULL);

	//Prepare OpenGL
	#if UseOpenGL
	if(GraphicsEngine==GE_OpenGL)
		if(!InitializeOpenGL(MainHwnd))
		{
			MessageBox(MainHwnd, "Can't initialize OpenGL, Defaulting to WIN32 GDI", "Error initializing OpenGL", MB_OK);
			GraphicsEngine=GE_WinGDI;
		}
	#else //If not using OGL, force GDI
		GraphicsEngine=GE_WinGDI;
	#endif

	//Draw the main window for the first time
	MainBackBuffer=CreateGraphicsBuffer(MainHwnd);
	ShowWindow(MainHwnd, SW_SHOW); //Calls CreateFractal and DrawWindow from WM_SIZE

	//Main loop listening for events
	MSG msg;
	ZeroMemory(&msg, sizeof(MSG));
	while(GetMessage(&msg, NULL, 0, 0))
	{
		if(IsDialogMessage(DialogHwnd, &msg))
			continue;

		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	//Destroy windows handles
	ReleaseGraphicsBuffer(MainBackBuffer);
	UnregisterClass("JuliaFractal", hInst);

	return S_OK;
}

GraphicsBuffer CreateGraphicsBuffer(HWND CopyHwnd) //Creates a graphics buffer
{
	GraphicsBuffer NewBuffer;

	#if UseOpenGL
	if(GraphicsEngine==GE_OpenGL)
	{
		glGenTextures(1, &NewBuffer.OGL);
		glBindTexture(GL_TEXTURE_2D, NewBuffer.OGL);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	}
	#endif

	#if UseWinGDI
	if(GraphicsEngine==GE_WinGDI)
	{
		HDC CopyDC=GetDC(CopyHwnd);
		NewBuffer.GDI=CreateCompatibleDC(CopyDC);
		ReleaseDC(CopyHwnd, CopyDC);
	}
	#endif

	SizeGraphicsBuffer(CopyHwnd, NewBuffer);
	return NewBuffer;
}

void ReleaseGraphicsBuffer(GraphicsBuffer GBuffer) //Releases a graphics buffer
{
	#if UseOpenGL
	if(GraphicsEngine==GE_OpenGL)
		glDeleteTextures(1, &GBuffer.OGL);
	#endif


	#if UseWinGDI
	if(GraphicsEngine==GE_WinGDI)
		DeleteDC(GBuffer.GDI);
	#endif
}

void SizeGraphicsBuffer(HWND CopyHwnd, GraphicsBuffer GBuffer) //Sizes a graphic buffer's surface to the needed size for the window
{

	#if UseOpenGL
	if(GraphicsEngine==GE_OpenGL)
	{
		glBindTexture(GL_TEXTURE_2D, GBuffer.OGL);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
	}
	#endif
	

	#if UseWinGDI
	if(GraphicsEngine==GE_WinGDI)
	{
		HDC CopyDC=GetDC(CopyHwnd);
		HBITMAP NewBitmap=CreateCompatibleBitmap(CopyDC, Width, Height);
		SelectObject(GBuffer.GDI, NewBitmap);
		DeleteObject(NewBitmap);
		ReleaseDC(CopyHwnd, CopyDC);
	}
	#endif
}

void DrawToWindow(HWND hWnd) //Draw the back buffer to the main screen
{
	//Draw the back buffer to the main screen
	HDC TheDC=GetDC(hWnd);

	//Draw using OGL
	#if UseOpenGL
	if(GraphicsEngine==GE_OpenGL)
	{
		//Draw the back buffer as a quad
		const BYTE QuadConfig[]={0,0,1,0,1,1,0,1}; //Quad x/y corners
		glBindTexture(GL_TEXTURE_2D, MainBackBuffer.OGL);
		glBegin(GL_QUADS);
		for(int i=0;i<4;i++) //Each vertex of the quad
		{
			glTexCoord2i(QuadConfig[i*2], 1-QuadConfig[i*2+1]);
			glVertex2i((int)QuadConfig[i*2]*Width, (int)QuadConfig[i*2+1]*Height);
		}
		glEnd();

		//Draw to the screen
		SwapBuffers(TheDC);
	}
	#endif

	//Draw using GDI
	#if UseWinGDI
	if(GraphicsEngine==GE_WinGDI)
		BitBlt(TheDC,0,0,Width,Height,MainBackBuffer.GDI,0,0,SRCCOPY);
	#endif

	ReleaseDC(hWnd, TheDC);
}

void AnimateZoom(double NewXMin, double NewXMax, double NewYMin, double NewYMax, HWND hWnd, int Frames) //Animate a zoom in/out. Takes in new boundary points, the window to draw to, and the number of frames to do the animation in
{
	//Determine zoomed area pixel information
	double ScaleX=XMax-XMin, ScaleY=YMax-YMin; //Determine the current window dimensions (scale) from boundary points
	double NewPixelXMin=(NewXMin-XMin)/ScaleX*Width, NewPixelYMin=(YMax-NewYMax)/ScaleY*Height; //The pixel coordinates for the upper left corner of the zoom section
	double NewPixelWidth=(NewXMax-NewXMin)/ScaleX*Width, NewPixelHeight=(NewYMax-NewYMin)/ScaleY*Height; //The pixel dimensions of the zoom section


	//Create buffers if animating
	GraphicsBuffer TempBuffer, CombineBuffer;
	if(AnimationEnabled)
	{
		//Move the current back buffer to a temporary buffer for use in animation
		TempBuffer=MainBackBuffer;
		MainBackBuffer=CreateGraphicsBuffer(hWnd);
		
		//If in windows, create a buffer for combining during animation
		#if UseWinGDI
		if(GraphicsEngine==GE_WinGDI)
			CombineBuffer=CreateGraphicsBuffer(hWnd);
		#endif
	}

	SetConfigBoundaries(NewXMin, NewXMax, NewYMin, NewYMax); //Send updated info to the dialog box

	//Copy over the new boundary points and draw the new fractal
	XMin=NewXMin;
	XMax=NewXMax;
	YMin=NewYMin;
	YMax=NewYMax;
	CreateFractal();

	//If animation is not enabled, exit prematurely
	if(!AnimationEnabled)
	{
		DrawToWindow(hWnd); //Draw the final result
		return;
	}

	//Do the animation
	HDC TheDC=GetDC(hWnd); //The front buffer to draw to
	for(int i=1;i<=Frames;i++)
	{
		//Determine the animation progress percent
		float CurrentProgress=(float)i/Frames;

		//Determine the new pixel position and dimensions for the new fractal
		double NewFractalWidth=(NewPixelWidth+(Width-NewPixelWidth)*CurrentProgress);
		double NewFractalHeight=(NewPixelHeight+(Height-NewPixelHeight)*CurrentProgress);
		double NewFractalX=(NewPixelXMin*(1-CurrentProgress));
		double NewFractalY=(NewPixelYMin*(1-CurrentProgress));

		//Determine how much the new fractal grew and moved
		double GrowX=NewFractalWidth/NewPixelWidth;
		double GrowY=NewFractalHeight/NewPixelHeight;
		double TranslateX=(NewFractalX-NewPixelXMin)/NewPixelWidth;
		double TranslateY=(NewFractalY-NewPixelYMin)/NewPixelHeight;

		//Save the 2 fractal levels to draw so it can be done in different orders
		struct DrawFractalLevel
		{
			int x, y, width, height;
			GraphicsBuffer Buffer;
			inline DrawFractalLevel(int x, int y, int width, int height, GraphicsBuffer Buffer) : x(x), y(y), width(width), height(height), Buffer(Buffer) {}
		};
		DrawFractalLevel F[2]={
			DrawFractalLevel( //Enlarge the old fractal in proportion to the new fractal
				(int)(Width*TranslateX),
				(int)(Height*TranslateY),
				(int)(Width*GrowX),
				(int)(Height*GrowY),
				TempBuffer
			),
			DrawFractalLevel( //Enlarge the new fractal
				(int)NewFractalX,
				(int)NewFractalY,
				(int)NewFractalWidth,
				(int)NewFractalHeight,
				MainBackBuffer
			)
		};

		//If using OpenGL, clear the back buffer first (hide a problem where certain parts of the screen are not drawn when zooming)
		#if UseOpenGL
		if(GraphicsEngine==GE_OpenGL)
		{
			glClearColor(0, 0, 1, 0);
			glClear(GL_COLOR_BUFFER_BIT);
		}
		#endif

		//Draw the 2 fractal levels in the proper order
		int SwapI=(GrowX<1)&1; //If zooming out, our grow is <1 . If this is the case,  we will be swapping all "i" instances below to draw in reverse order
		for(int i=0;i<2;i++)
		{
			DrawFractalLevel *C=&F[i^SwapI]; //The current fractal level to draw

			#if UseOpenGL
			if(GraphicsEngine==GE_OpenGL)
			{
				const BYTE QuadConfig[]={0,0,1,0,1,1,0,1}; //Quad x/y corners
				glBindTexture(GL_TEXTURE_2D, C->Buffer.OGL);
				glBegin(GL_QUADS);
				for(int i=0;i<4;i++) //Each vertex of the quad
				{
					glTexCoord2i(QuadConfig[i*2], 1-QuadConfig[i*2+1]);
					glVertex2i(C->x+(int)QuadConfig[i*2]*C->width, C->y+(int)QuadConfig[i*2+1]*C->height);
				}
				glEnd();
			}
			#endif

			#if UseWinGDI
			if(GraphicsEngine==GE_WinGDI)
				StretchBlt(CombineBuffer.GDI, C->x, C->y, C->width, C->height, C->Buffer.GDI, 0, 0, Width, Height, SRCCOPY);
			#endif
		}

		//Output the combined result
		#if UseOpenGL
		if(GraphicsEngine==GE_OpenGL)
			SwapBuffers(TheDC);
		#endif
		#if UseWinGDI
		if(GraphicsEngine==GE_WinGDI)
			BitBlt(TheDC, 0, 0, Width, Height, CombineBuffer.GDI, 0, 0, SRCCOPY);
		#endif
	}

	//Delete the temporary buffers
	ReleaseDC(hWnd, TheDC);
	ReleaseGraphicsBuffer(TempBuffer);
	if(GraphicsEngine==GE_WinGDI)
		ReleaseGraphicsBuffer(CombineBuffer);

	DrawToWindow(hWnd); //Draw the final result
}

void CreateFractal() //Draws the fractal into the MainBackBuffer
{
	//Create the Fractal
	DWORD *FractalArray=new DWORD[Width*Height]; //Create the pixel array
	DrawFractal(FractalArray); //Do the drawing

	//Output to MainBackBuffer
	#if UseOpenGL
	if(GraphicsEngine==GE_OpenGL)
	{
		glBindTexture(GL_TEXTURE_2D, MainBackBuffer.OGL);
		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, FractalArray);
	}
	#endif
	#if UseWinGDI
	if(GraphicsEngine==GE_WinGDI)
	{
		BITMAPINFO BitmapInfo={sizeof(BITMAPINFOHEADER), Width, Height, 1, 32, BI_RGB, 0, 0, 0, 0, 0};
		SetDIBits(MainBackBuffer.GDI, (HBITMAP)GetCurrentObject(MainBackBuffer.GDI, OBJ_BITMAP), 0, Height, FractalArray, &BitmapInfo, DIB_RGB_COLORS);
	}
	#endif

	//Clear out pixel array
	delete[] FractalArray;
}

void DrawFractal(DWORD *PixelArray) //Does the actual fractal drawing into a pixel array
{
	SetWindowText(TheMainHwnd, "Julia Fractal [Computing Level]");
	for(int Y=0;Y<Height;Y++)
		for(int X=0;X<Width;X++)
		{
			//Determine the point in the 2D fractal plane we are checking and assign to {x,y} (used for break out testing) and {x0,y0} (which is what x/y are updated by on each iteration)
			double x=(double)(XMax-XMin)*((double)X/Width)+XMin, y=(YMax-YMin)*((double)Y/Height)+YMin, x0=(JUseC ? x : 0)+JReal, y0=(JUseC ? y : 0)+JImag;

			//Determine if and when the point breaks out
			for(int iteration=0; x*x+y*y<=(2*2) && iteration<MaxIterations; iteration++)
			{
				double xtemp=x*x-y*y+x0;
				y=2*x*y+y0;
				x=xtemp;
			}

			//Assign the pixel color depending on if and when the point breaks out
			PixelArray[Y*Width+X]=(iteration == MaxIterations ? 0 : ProcessColor(iteration, x, y, 2, 2));
		}
	SetWindowText(TheMainHwnd, "Julia Fractal");
}

DWORD ProcessColor(int IterationsToEscape, double ValueAtEndReal, double ValueAtEndImaginary, double BailoutRadius, double Power) //Determine the color of a pixel based on its state - Parameters = n, Zn{r}, Zn{i} B, P
{
	double Percent=log(2*log(BailoutRadius))-log(log(sqrt(fabs(ValueAtEndReal*ValueAtEndReal+ValueAtEndImaginary*ValueAtEndImaginary))))/log(Power); //A linear interpolated percent between escape states
	double State=(double)IterationsToEscape+Percent; //Current actual escape state plus its interpolated percent

	//Colors for each range
	const DWORD ColorRanges[]={
		0x000080, //Dark Blue
		0x0000FF, //Blue
		0xA0A0FF, //Light Blue
		0xFFFF80, //Light yellow
		0x80FF80, //Light green
		0x008000, //Dark green
		0x008080, //Forest green
		0x00FFFF, //Cyan
		0xFF00FF, //Purple
		0xFF0000, //Red
		0x800000, //Dark red
		0x000080, //Dark blue - Start back at the beginning
	};
	const BYTE NumColorRanges=sizeof(ColorRanges)/sizeof(DWORD)-1;
	const BYTE RangeSize=20; //How many escape states each color ranges encompasses

	//Determine which color range the state is in and interpolate it into that range
	DWORD PixelRangeNum=(int)State/RangeSize;
	double PercentInRange=(State-RangeSize*PixelRangeNum)/RangeSize;
	const BYTE *StartC=(BYTE*)&ColorRanges[PixelRangeNum%NumColorRanges], *EndC=StartC+sizeof(DWORD);

	DWORD Color=0;
	for(int i=0;i<3;i++)
		Color|=(StartC[i]+(int)((EndC[i]-StartC[i])*PercentInRange))<<(i*8);

	//Return color normally for GDI
	if(GraphicsEngine==GE_WinGDI)
		return Color;

	//Color has to be swapped for OGL (RGB TO BGR)
	_asm mov eax,Color
	_asm bswap eax
	_asm shr eax,8
	_asm mov Color,eax
	return Color;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) //Process windows events
{
	switch(msg)
	{
		case WM_PAINT:
		{
			//Draw the backbuffer to the main screen
			DrawToWindow(hWnd);
			break;
		}
		case WM_SIZE:
		{
			Width=LOWORD(lParam);
			Height=HIWORD(lParam);

			//Set the window dimensions for OpenGL
			#if UseOpenGL
			if(GraphicsEngine==GE_OpenGL)
			{
				ReleaseGraphicsBuffer(MainBackBuffer);
				InitializeOpenGL(hWnd);
				MainBackBuffer=CreateGraphicsBuffer(hWnd);
				glMatrixMode(GL_PROJECTION);
				glLoadIdentity();
				glOrtho(0, Width, Height, 0, 0, 1);
			}
			#endif

			SetConfigWindowDimensions(Width, Height); //Send updated info to the dialog box
			SizeGraphicsBuffer(hWnd, MainBackBuffer); //Resize the back buffer

			//Redraw the window
			CreateFractal();
			DrawToWindow(hWnd);
			break;
		}
		case WM_LBUTTONDOWN:
		{
			//Zoom in by 200% around point
			double ScaleX=XMax-XMin, ScaleY=YMax-YMin; //Determine the current window dimensions

			//Determine the clicked point in the window
			double PointX=(double)LOWORD(lParam)/Width*ScaleX+XMin;
			double PointY=YMax-(double)HIWORD(lParam)/Height*ScaleY;

			AnimateZoom(PointX-ScaleX/8, PointX+ScaleX/8, PointY-ScaleY/8, PointY+ScaleY/8, hWnd, AnimationFrames); //Zoom in
			break;
		}
		case WM_RBUTTONDOWN:
		{
			//Zoom out by 200%
			double ScaleX=XMax-XMin, ScaleY=YMax-YMin; //Determine the current window dimensions

			//Determine the center of the window
			double PointX=.5*ScaleX+XMin;
			double PointY=YMax-.5*ScaleY;

			AnimateZoom(PointX-ScaleX*2, PointX+ScaleX*2, PointY-ScaleY*2, PointY+ScaleY*2, hWnd, AnimationFrames); //Zoom out
			break;
		}
		case WM_CHAR:
			if(wParam=='=') //Detail level up
				SetConfigDetailLevel(MaxIterations*=2); //Send updated info to the dialog box
			else if(wParam=='-') //Detail level down
				SetConfigDetailLevel(MaxIterations/=2); //Send updated info to the dialog box
			else if(wParam==13) //Enter=Open config window
			{
				//Make sure dialog isn't already open
				if(IsDialogOpen())
					break;

				//Open dialog and set all its text windows
				DialogHwnd=CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(101), hWnd, ConfigProc);
				SetConfigBoundaries(XMin, XMax, YMin, YMax);
				SetConfigWindowDimensions(Width, Height);
				SetConfigDetailLevel(MaxIterations);
				SetInitDialogInfo(AnimationEnabled, JUseC, JReal, JImag, GraphicsEngine==GE_OpenGL);

				//Disable OpenGL Button if not used
				#if !UseOpenGL
					ConfigDisableOGL();
				#endif

			}
			else //Assumes a direction key was pressed
			{
				//Move in given direction
				double ScaleX=XMax-XMin, ScaleY=YMax-YMin; //Determine the current window dimensions

				//Determine the center of the window
				double PointX=.5*ScaleX+XMin;
				double PointY=YMax-.5*ScaleY;

				if(wParam=='w')
					PointY+=ScaleY/4;
				else if(wParam=='a')
					PointX+=ScaleX/-4;
				else if(wParam=='s')
					PointY+=ScaleY/-4;
				else if(wParam=='d')
					PointX+=ScaleX/4;
				else //If not w/a/s/d, do nothing
					break;

				AnimateZoom(PointX-ScaleX/2, PointX+ScaleX/2, PointY-ScaleY/2, PointY+ScaleY/2, hWnd, AnimationFrames/10); //Slide the window
				break;
			}

			//Redraw the window when changing detail level
			CreateFractal();
			DrawToWindow(hWnd);
			break;
		case WM_MOUSEMOVE:
		{
			if(!IsDialogOpen())
				break;

			//Get Pixel Coordinates
			int X=LOWORD(lParam), Y=HIWORD(lParam);

			//Determine the point in the 2D fractal plane - see "DrawFractal" function
			double x=(double)(XMax-XMin)*((double)X/Width)+XMin, y=(YMax-YMin)*((double)Y/Height)+YMin, x0=(JUseC ? x : 0)+JReal, y0=(JUseC ? y : 0)+JImag;
			double XRem=x, YRem=y; //Remember coordinate points

			//Determine if and when the point breaks out - see "DrawFractal" function
			for(int iteration=0; x*x+y*y<=(2*2) && iteration<MaxIterations; iteration++)
			{
				double xtemp=x*x-y*y+x0;
				y=2*x*y+y0;
				x=xtemp;
			}

			//Determine the color
			DWORD Color=(iteration==MaxIterations ? 0 : ProcessColor(iteration, x, y, 2, 2));
			if(GraphicsEngine==GE_OpenGL) //Swap the color back to RGB
			{
				_asm mov eax,Color
				_asm bswap eax
				_asm shr eax,8
				_asm mov Color,eax
			}

			SetConfigPixelInfo(X, Y, XRem, YRem, iteration+(iteration==MaxIterations ? 0 : log(2*log(2))-log(log(sqrt(fabs(x*x+y*y))))/log(2)), Color); //See the "ProcessColor" function for the EscapeV parameter
			break;
		}
		case WM_DESTROY:
			//Terminate the program
			PostQuitMessage(0);
			return 0;
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//Initialize OpenGL and Back Buffer
#if UseOpenGL
bool InitializeOpenGL(HWND TheHwnd)
{
	//If OpenGL is initialized, the old context needs to be deleted. This needs to be done so a new back buffer can be created.
	static bool GLInitialized=false;
	static HGLRC MainRC;
	if(GLInitialized)
	{
		wglMakeCurrent(NULL,NULL);
		wglDeleteContext(MainRC);
	}

	//Set the Pixel Format of the window
	HDC MainDC=GetDC(TheHwnd);
	const PIXELFORMATDESCRIPTOR pfd={sizeof(PIXELFORMATDESCRIPTOR),1,PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,PFD_TYPE_RGBA,32, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, PFD_MAIN_PLANE, 0, 0, 0, 0};
	GLuint PixelFormat=ChoosePixelFormat(MainDC,&pfd);
	if(!PixelFormat || !SetPixelFormat(MainDC,PixelFormat,&pfd)) //Error out if can't be set
		return false;

	//Create the GL Context
	MainRC=wglCreateContext(MainDC);
	if(!MainRC)
		return false;
	if(!wglMakeCurrent(MainDC,MainRC))
	{
		wglMakeCurrent(NULL,NULL);
		wglDeleteContext(MainRC);
		return false;
	}
	ReleaseDC(TheHwnd, MainDC);
	glDisable(GL_DEPTH_TEST); //No depth testing in 2D
	glEnable(GL_TEXTURE_2D); //Always use textures
	
	GLInitialized=true;
	return true;
}
#endif

//Receive back changed information from the configuration dialog
void DialogReturnInformation(double x1, double x2, double y1, double y2, int x, int y, int d, bool AnimationEnabledLocal, bool UseC, double Real, double Imag, bool UsingOGL)
{
	//First change all fractal display values that do not trigger buffer 
	XMin=x1;
	XMax=x2;
	YMin=y1;
	YMax=y2;
	MaxIterations=d;
	AnimationEnabled=AnimationEnabledLocal;
	JUseC=UseC;
	JReal=Real;
	JImag=Imag;

	//Change graphics engine
	bool GraphicsEngineChange=(UsingOGL!=(GraphicsEngine==GE_OpenGL));
	if(GraphicsEngineChange)
	{
		ReleaseGraphicsBuffer(MainBackBuffer);
		GraphicsEngine=(UsingOGL ? GE_OpenGL : GE_WinGDI);
	}

	bool WindowResize=(x!=Width || y!=Height);
	if(WindowResize) //Check for height change
	{
		//Determine the size of the actual window that we want based upon client area dimensions
		RECT WindowRect={0, 0, x, y};
		AdjustWindowRect(&WindowRect, WS_OVERLAPPEDWINDOW, false);
		Width=(WORD)(WindowRect.right-WindowRect.left);
		Height=(WORD)(WindowRect.bottom-WindowRect.top);
		SetWindowPos(TheMainHwnd, NULL, 0, 0, Width, Height, SWP_NOZORDER|SWP_NOMOVE);
	}

	if(GraphicsEngineChange || WindowResize) //Trigger a window size change to refresh all buffers
	{
		SendMessage(TheMainHwnd, WM_SIZE, 0, Width|(Height<<16));
		return;
	}
	else //Otherwise, just redraw the fractal
	{
		CreateFractal();
		DrawToWindow(TheMainHwnd);
	}
}